@@ -1,7 +1,7 @@
 # Automated tests for the `coloredlogs' package.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: January 17, 2018
+# Last Change: February 14, 2020
 # URL: https://coloredlogs.readthedocs.io
 
 """Automated tests for the `coloredlogs` package."""
@@ -362,6 +362,13 @@ class ColoredLogsTestCase(TestCase):
         logging.info("This should include milliseconds.")
         assert all(map(PATTERN_INCLUDING_MILLISECONDS.match, stream.getvalue().splitlines()))
 
+    def test_support_for_milliseconds_directive(self):
+        """Make sure milliseconds using the ``%f`` directive are supported."""
+        stream = StringIO()
+        install(reconfigure=True, stream=stream, datefmt='%Y-%m-%dT%H:%M:%S.%f%z')
+        logging.info("This should be timestamped according to #45.")
+        assert re.match(r'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}[+-]\d{4}\s', stream.getvalue())
+
     def test_plain_text_output_format(self):
         """Inspect the plain text output of coloredlogs."""
         logger = VerboseLogger(random_string(25))
@@ -379,6 +386,14 @@ class ColoredLogsTestCase(TestCase):
                                  (logger.warning, 'WARNING'),
                                  (logger.error, 'ERROR'),
                                  (logger.critical, 'CRITICAL')):
+            # XXX Workaround for a regression in Python 3.7 caused by the
+            # Logger.isEnabledFor() method using stale cache entries. If we
+            # don't clear the cache then logger.isEnabledFor(logging.DEBUG)
+            # returns False and no DEBUG message is emitted.
+            try:
+                logger._cache.clear()
+            except AttributeError:
+                pass
             # Prepare the text.
             text = "This is a message with severity %r." % severity.lower()
             # Log the message with the given severity.
@@ -476,14 +491,14 @@ class ColoredLogsTestCase(TestCase):
         # Sanity check that log messages aren't enabled by default.
         with CaptureOutput() as capturer:
             os.environ['COLOREDLOGS_AUTO_INSTALL'] = 'false'
-            subprocess.call(command_line)
+            subprocess.check_call(command_line)
             output = capturer.get_text()
         assert needle not in output
         # Test that the $COLOREDLOGS_AUTO_INSTALL environment variable can be
         # used to automatically call coloredlogs.install() during initialization.
         with CaptureOutput() as capturer:
             os.environ['COLOREDLOGS_AUTO_INSTALL'] = 'true'
-            subprocess.call(command_line)
+            subprocess.check_call(command_line)
             output = capturer.get_text()
         assert needle in output
 
